XIncludeFile "..\include\Chipmunk4PB_v0.4.pbi"
Global *space.cpSpace
Global *staticBody.cpBody
Global ticks
Global NewList myContact.cpVect()
Global NewList myContact1.cpVect()
Procedure DrawPolyShape(*poly.cpPolyShape)
  Protected *verts.cpVect=*poly\verts
  Protected *last.cpVect
  Protected v.cpVect
  Protected first.cpVect
  Protected last.cpVect
  
  *last=*verts+SizeOf(cpVect)*(*poly\numVerts-1)
  cpvrotate(v, *last, *poly\shape\body\rot)
  cpvadd(v, v, *poly\shape\body\p)
  cpv(last, v\x, v\y)
  
  For i=0 To *poly\numVerts-1
    cpvrotate(v, *verts, *poly\shape\body\rot)
    cpvadd(v, v, *poly\shape\body\p)
    LineXY(last\x, last\y, v\x, v\y, RGB(0, 0, 0))
    cpv(last, v\x, v\y)
    *verts+SizeOf(cpVect)
  Next
  Circle(*poly\shape\body\p\x, *poly\shape\body\p\y, 2,  #Red)
EndProcedure
Procedure DrawCircleShape(*circle.cpCircleShape)
  Protected x=*circle\tc\x
  Protected y=*circle\tc\y
  Protected r.f=*circle\r
  Protected a.f=*circle\shape\body\a
  Circle(x, y, r)
  LineXY(x, y, x+Cos(a)*r, y+Sin(a)*r)
EndProcedure
Procedure DrawSegmentShape(*seg.cpSegmentShape)
  Protected a.cpVect
  Protected b.cpVect
  
  cpvrotate(a, *seg\a, *seg\shape\body\rot)
  cpvrotate(b, *seg\b, *seg\shape\body\rot)
  cpvadd(a, a, *seg\shape\body\p)
  cpvadd(b, b, *seg\shape\body\p)
  
  LineXY(a\x, a\y, b\x, b\y, RGB(0, 0, 0))
EndProcedure
ProcedureC DrawShape(*shape.cpShape, *datas)
  Select *shape\klass\type
    Case #CP_CIRCLE_SHAPE
      DrawCircleShape(*shape)
    Case #CP_SEGMENT_SHAPE
      DrawSegmentShape(*shape)
    Case #CP_POLY_SHAPE
      DrawPolyShape(*shape)
  EndSelect
EndProcedure
ProcedureC DrawCollisions()
  ForEach myContact()
    Circle(myContact()\x, myContact()\y, 2, #Blue)
  Next
  ClearList(myContact())
  ForEach myContact1()
    Circle(myContact1()\x, myContact1()\y, 2, #Green)
  Next
  ClearList(myContact1())
EndProcedure
ProcedureC FixBodyPositions(*body.cpBody, *datas)
  ;check if object is offscreen
  If *body\data <> 10
    If *body\p\y>500 Or *body\p\x>680 Or *body\p\x<-20
      ;reset its position if necessary
      *body\p\x=Random(640)
      *body\p\y=-20*Random(30)
    EndIf
  EndIf
EndProcedure


Procedure InitObjects()
  Protected *shape.cpShape
  Protected *body.cpBody
  Protected i, j
  Protected offset.cpVect
  Protected stagger.f
  
  cpInitChipmunk()
  cpResetShapeIdCounter()
  
  ;{/// PHYSIC ENVIRONMENT
  *staticBody=cpBodyNew(#INFINITY, #INFINITY)
  
  *space=cpSpaceNew()
  *space\iterations=500
  *space\damping = 0.3
  cpv(*space\gravity, 0, 900)
  cpSpaceResizeStaticHash(*space, 30.0, 10)
  cpSpaceResizeActiveHash(*space, 20.0, 10)
  ;}
  
  ;{/// TRIANGLES
  Dim triangle.cpVect(3)
  cpv(triangle(0), 20, 20)
  cpv(triangle(1), 0, -10)
  cpv(triangle(2), -20, 20)
  
  For i=0 To 8
    For j=0 To 5
      s.l=j%2
      stagger=s*40
      cpv(offset, i*80+stagger, 100+(j*70) )
      *body.cpBody=cpBodyNew(1.0, cpMomentForPoly(1.0, ArraySize(triangle()), @triangle(), @cpvzero))
      *body\data = 10;
      cpv(*body\p,i*80+stagger, 100+(j*70))
      cpSpaceAddBody(*space, *body);     
      *shape=cpPolyShapeNew(*body, ArraySize(triangle()), @triangle(), @cpvzero)         
      *shape\e=1.0
      *shape\u=1.0        
      *shape\collision_type = 1
      cpSpaceAddShape(*space, *shape)
      
      cpSpaceAddConstraint(*space, cpPivotJointNew( *body,*staticBody,@offset ));
    Next
  Next
  
  ;{/// FALLING COINS
  Dim COIN.cpVect(5)
  For i=0 To ArraySize(COIN())-1
    Protected angle.f=-2*#PI*i/ArraySize(COIN())
    cpv(COIN(i), 15*Cos(angle), 15*Sin(angle))
  Next
  ;   
  For i=0 To 100
    *body=cpBodyNew(1.0, cpMomentForPoly(1.0, ArraySize(COIN()), @COIN(), @cpvzero))
    cpv(*body\p, Random(640), -20*Random(200) )     
    cpSpaceAddBody(*space, *body)
    *shape=cpPolyShapeNew(*body, ArraySize(COIN()), @COIN(), @cpvzero)
    *shape\e=0.0
    *shape\u=0.0
    *shape\collision_type = 0
    cpSpaceAddShape(*space, *shape)
  Next
  
EndProcedure

Procedure MoveObjects()
  ticks+1
  Protected steps.f=1.0
  Protected dt.f=1.0/60.0/steps
  Protected i
  For i=0 To steps
    cpSpaceStep(*space, dt)
    cpSpaceEachBody(*space, @FixBodyPositions(), #Null)
  Next   
EndProcedure
Procedure DrawObjects()
  ClearScreen(RGB(255, 255, 255))
  StartDrawing(ScreenOutput())
  DrawingMode(#PB_2DDrawing_Outlined)
  cpSpaceHashEach(*space\activeShapes, @DrawShape(), #Null)
  cpSpaceHashEach(*space\staticShapes, @DrawShape(), #Null)
  DrawCollisions()
  DrawingMode(#PB_2DDrawing_Default)
  StopDrawing()
  FlipBuffers()   
EndProcedure
Procedure FreeObjects()
  cpSpaceFree(*space)
  cpBodyFree(*staticBody)
EndProcedure

InitSprite()
InitKeyboard()
InitObjects()
;/ ---------------------------------------------------------------------------------------------------------------------
ProcedureC col_preSolve(*arb.cpArbiter, *space.cpSpace, *pData)
  AddElement(myContact())
  
  cpArbiterGetPoint(myContact(),*arb, 0)
  ProcedureReturn #True
EndProcedure

ProcedureC col_preSolve1(*arb.cpArbiter, *space.cpSpace, *pData)
  AddElement(myContact1())
  cpArbiterGetPoint(myContact1(),*arb, 0)
  ProcedureReturn #True
EndProcedure

cpSpaceAddCollisionHandler(*space, 1, 0, #Null, @col_preSolve(), #Null, #Null, #Null)
cpSpaceAddCollisionHandler(*space, 0, 0, #Null, @col_preSolve1(), #Null, #Null, #Null)
;/ ---------------------------------------------------------------------------------------------------------------------

OpenWindow(100, 0, 0, 640, 480, "Chipmunk4PB CollisionsHandler Demo", #PB_Window_ScreenCentered | #PB_Window_SystemMenu)
OpenWindowedScreen(WindowID(100), 0, 0, 640, 480, 0, 0, #PB_Screen_WaitSynchronization )
Repeat
  DrawObjects()
  MoveObjects()
  ExamineKeyboard()
Until KeyboardPushed(#PB_Key_Escape) Or WaitWindowEvent(1)=#PB_Event_CloseWindow
FreeObjects()
End


; IDE Options = PureBasic 4.60 Beta 1 (Windows - x86)
; CursorPosition = 108
; FirstLine = 69
; Folding = ---
; EnableXP
; Executable = constraint.exe
; DisableDebugger